#include <Mega32.h>  
#include <stdio.h> 	// sprintf amungst other definitions
#include "project.h"
         
/*********************************************************/
//UART character-ready ISR
interrupt [USART_RXC] void uart_rec(void){

	r_char=UDR;    //get a char
	//UDR=r_char;    //then print it 
	
	//build the input string
	if (r_char != '\r')
		r_buffer[r_index++]=r_char;
	else{
		putchar('\n');  			//use putchar to avoid overwrite
		r_buffer[r_index]=0x00;    		//zero terminate
		r_ready=1;				//signal cmd processor
		UCSRB.7=0;   				//stop rec ISR 
	}
}

/**********************************************************/
//UART xmit-empty ISR
interrupt [USART_DRE] void uart_send(void){

	t_char = t_buffer[++t_index];
    if(t_char == 0){
	
		UCSRB.5=0; //kill isr 
		t_ready=1; //transmit done 
	}
	else	
	UDR = t_char ;     //send the char 
}  

/**********************************************************/
// Timer 0 compare ISR                                       
interrupt [TIM0_COMP] void timer0_compare(void){
		  
	// increment .5ms time variables
	time0++;    
	time1++;           
 	  
}

/**********************************************************/       
//Entry point and task scheduler loop
void main(void){  
      
	// temp vars
	BYTE i, prev_time;
	UINT temp;
                  
	time0 = 0;  
	time1 = 0;

	Port_Init();  			// set up port for proper i/o
	Timer0_Init();          // set up timer 0  
	ACD0_Init();			// set up ADC and start first conversion                             
	UART_Init();            // set up UART
	
	PORTD.7 = 0;			// enable analog mux's
	        
	// enable interrupts
	#asm
		sei
	#endasm   
  
  	// spin forever
  	while(1){
		              
		// 2.5 ms count tick
		if( time0 == 5 ){ 
		
	 		time0 = 0; 
		     
			// sample each channel continuously
			for( i=0; i<NUM_CHANNELS; i++ ){
			
				// change channel
				ADMUX = muxvals[i];

	   			// start new conversion 
				ADCSRA.6=1;
				
				// poll for completion
				while( ADCSRA.6 == 1 );
				               
				// grab data
				//data[i] += ADCH; 
				temp = (ADCL>>6);
				temp |= ( (UINT)ADCH ) << 2;
				data[i] += temp;
		   	}

		}
		 
		// 10 ms count tick, send the data
		if( time1 == 20 ){ 
		
			time1 = 0;
		
			send_data(); 
			
			data[0] = 0;
			data[1] = 0;
			data[2] = 0;
			data[3] = 0;
			data[4] = 0;
			data[5] = 0;	
		
		}
  
  	}// end while(1)    
} 

//--------------------------------------------------------
//	SEND_DATA
//-------------------------------------------------------- 
// send x,y,z data out to UART
void send_data(){   

	BYTE i, temp;

	for( i=0; i<NUM_CHANNELS; i++ )
	 	data[i] = data[i]>>2;
	
                              
	// send data out serially 
	t_buffer[0] = 0x01;
	t_buffer[1] = (BYTE)(data[0]&0x00FF);	// data[0] lb -- x1
	t_buffer[2] = (BYTE)(data[1]&0x00FF);	// data[1] lb -- y1
	t_buffer[3] = (BYTE)(data[2]&0x00FF);	// data[2] lb -- z1	 
	
 	t_buffer[4] = (BYTE)(data[3]&0x00FF);	// data[0] lb -- x2
	t_buffer[5] = (BYTE)(data[4]&0x00FF);	// data[1] lb -- y2
	t_buffer[6] = (BYTE)(data[5]&0x00FF);	// data[2] lb -- z2               
	
	temp =  (data[0]>>2)&0b11000000;
	temp |= (data[1]>>4)&0b00110000;
	temp |= (data[2]>>6)&0b00001100;
	temp |= (data[3]>>8)&0b00000011;
	t_buffer[7] = temp;
	
	temp =  (data[4]>>2)&0b11000000;
	temp |= (data[5]>>4)&0b00110000;	
	t_buffer[8] = temp;
	
	t_buffer[9] = 0x00;
	
	/*
	t_buffer[8] = 
	
	t_buffer          
          
	t_buffer[4] =  ( (BYTE)(data[0]>>4) )&0x70;	// put next 3 msbs of data[0] in upper nibble of buffer4 -- x1
	t_buffer[4] |= ( (BYTE)(data[1]>>8) )&0x07;	// put next 3 msbs of data[1] in lower nibble of buffer4 -- y1
	
	t_buffer[5] =  ( (BYTE)(data[2]>>4) )&0x70;	// put next 3 msbs of data[2] in upper nibble of buffer4 -- z1
	t_buffer[5] |= ( (BYTE)(data[5]>>8) )&0x07;	// put next 3 msbs of data[5] in lower nibble of buffer4 -- z2	       
	       

	                          
	t_buffer[9] =  ( (BYTE)(data[3]>>4) )&0x70;	// put next 3 msbs of data[0] in upper nibble of buffer4 -- x2
	t_buffer[9] |= ( (BYTE)(data[4]>>8) )&0x07;	// put next 3 msbs of data[1] in lower nibble of buffer4 -- y2	
	
	t_buffer[10] = 0x02;
   	               
	t_buffer[11] = 0x00;
   */
	// send the data
	puts_int(); 
}

//---------------------------------------------------------
//	ADC0_Init
//---------------------------------------------------------
// Set up ADC with right adjusted values
// Start first conversion
void ACD0_Init(void){

	ADMUX =  0b01100000;		// using AVCC as reference
								// /w left adjusted values
								// looking @ ADC0
	ADCSRA = 0b10000111;		// enable ADC with interrupt on
								// auto trigger off, interrupt enable
								
	// start first conversion
  	ADCSRA.6 = 1; 
}

//---------------------------------------------------------
//	Port_Init
//---------------------------------------------------------
// Set up I/O PORTS:
// PORTA: input for Analog Signals
void Port_Init(void){

	DDRA = 0x00;	// port A as an input
	PORTA = 0xff;	// turn pull-ups on 
	            
	// port C,D as digital output
	DDRC = 0xff;
	DDRD = 0xff;
}

//---------------------------------------------------------
//	Timer0_Init
//---------------------------------------------------------
// Set up timer 0 with interrupt ticks @ .5ms
void Timer0_Init(void){  

	TCCR0 = 0b00001011;		// clock source @ /64 internal clock
	OCR0 = 125;             // compare value of 250
	TIMSK = 0x02;	        // enable compare match
} 

//---------------------------------------------------------
//	UART0_Init
//---------------------------------------------------------
// Set up Uart @ 9600 baud
void UART_Init(void){
    
  	UCSRB = 0x18 ;
  	UBRRL = 103 ;
  	
  	//init the task timers   
	r_ready=0;
  	t_ready=1; 

} 

//********************************************************** 
//  -- non-blocking keyboard check initializes ISR-driven
// receive. This routine merely sets up the ISR, which then
//does all the work of getting a command.
void gets_int(void){

  	r_ready=0;
  	r_index=0;
  	UCSRB.7=1;	// enables UART_RXC interrupt
}

//********************************************************** 
//  -- nonblocking print: initializes ISR-driven
// transmit. This routine merely sets up the ISR, then
//send one character, The ISR does all the work.   
void puts_int(void){

  	t_ready=0;
  	t_index=0;
  	if (t_buffer[0]>0){
  		putchar(t_buffer[0]);
  		UCSRB.5=1;	// enables UART_DRE interrupt
 	} 
}